# YASARA MOVIE
# TOPIC:    1. General
# TITLE:    1.6. Virtual reality (PC)
# REQUIRES: Model 23.4.1
# AUTHOR:   Kornel Ozvoldik, Elmar Krieger, Elina Krieger
# LICENSE:  GPL
# DESCRIPTION: A tutorial explaining how to work with YASARA in VR on PCs. Android is covered in a separate tutorial.

Renderer Vulkan
# Change working directory to the place of this macro
# (so that we don't have to specify paths when loading stuff).
CD (MacroDir)
OnError Continue
BallStickRadius 50,50
StickRadius 50
AtomSize 100
LightSource Alpha=30,Gamma=40,Ambience=085,Shadow=065
# Fresh start
Clear
# Left handed coordinate system
CoordSys Left
Projection perspective
UserInput Off
# Don't show console and head-up display
HUD Off
Console Off
MarkAtom None
# Load background
bgr=LoadJPG space.jpg
opt=0
# List of sections
sectionlist()=
"Section 1: VR setup",
"Section 2: VR controllers",
"Section 3: VR screen",
"Section 4: VR playground",
"Section 5: VR keyboard",
"Section 6: HIV protease",
"Section 7: The screening dome",
"Section 8: Teleportation",
"Continue to VR simulations"

Start:
DelAll
HideImage all
HideMessage
ShowImage (bgr)
# Load restriction enzyme PVuII
pvuobj = LoadPDB pvu2
Style Ball
HideAtom SideChain NucAcid
for col in 'H,320','C,330','N,Blue','O,340','S,330','P,340'
  ColorAtom Obj (pvuobj) Element (col)
# Create YASARA logo frame around it
# The logo frame created by ShowLogoObj 189 A x 68.4 A tall and 9 A thick. 
lenx=189.
leny=68.4e0
lenz=9.
# Build the logo frame from individual Xenon atoms to enable collisions during MD
ShowLogoObj (pvuobj),0
# Create the 'YASARA' letters
textlist="YAS  RA","V"
for i=1 to 2
  txtobj(i) = MakeTextObj YASARA,Width=200,Height=25
  PosText 50%,0%,justify=center
  Font Baikal,Height=100%,Color=White,Depth=2,DepthCol=404040
  Print (textlist(i))
PrintCon
MoveObj (txtobj2),X=8
MoveObj (join txtobj),Y=-5,Z=-45
Move Y=50,Z=250
RotateObj (txtobj2),X=180
ShowTiles 40,2,15,15,1.5,TileBottomLeft=ebfffe,97bda8,bae5d1,ffffff,Alpha=50%,SameCol=Yes,Joint=1.5,JointBottomLeft=779686,b9e3cf,456153,2a372d,JointSide=16211b
RotateObj Tiles,90
MoveObj Tiles,312,-45,100
# Load Yami and HMD
LoadYami
vpos()=PosObj (txtobj2)
PosOriObj HMD,(vpos),0,0,0
MoveObj HMD,0,100
MoveObj Yami,100
# Let logo fly in
for i=1 to 3
  objpos()=PosObj (i)
  MoveObj (i),0,0,-300
  AutoPosObj (i),(objpos),240,0
# Yami jumps into the scene
for i=0 to 240
  yax=-0.5*i+120
  step=i%60
  AnimateYami yax,90,1.,step
  Wait 1
for i=0 to 60
  AnimateYami 0,90+i*1.5,1.,i
  Wait 1
# HMD drops down
yamipos()=PosObj Yami
AutoPosObj HMD,(vpos1),(vpos2+18),(vpos3),20,1
AutoOriObj (txtobj2),0,0,0,50,0
AutoPosOriObj HMD,(vpos1),(vpos2+25),(vpos3-20),180,0,0,20,1
AutoPosOriObj HMD, X=1.601, Y=(4.441+yamipos2), Z=(8.704+yamipos3), Alpha=8.418, Beta=11.979, Gamma=-3.742,30,1
AutoMoveObj Tiles,Y=-0.5
AutoRotateObj Tiles,X=-0.01
vron=Stereo
Wait 100
DelObj Tiles
# Rotate scene around
AutoMoveObj Tiles,Y=-1
AutoPosOri 0,30,50,0,-180,0,Steps=90,Wait=No
Wait 45
DelObj Tiles
Wait 45
DelObj not HMD Yami
AutoPosOri 25,0,70,0,30,0,15,1

Menu:
# Reset scene
UserInput Off
HUD Off
MarkAtom None
DelAll
HideImage all
ShowImage (bgr)
LoadYami
PosOri 25,0,70,0,30,0
colorlist()='0000ff','0020ff','0040ff','0060ff','0070ff','0080ff','00a0ff','00c0ff','00ffff'
sections=count sectionlist
sectionsize=6
textobj = MakeTextObj Name=Sections,Width=60,Height=((sections-1)*sectionsize)
z=100
# Build menu
for i=1 to sections
  y=((0.5*(sections+1)-i)*sectionsize+1)
  color=colorlist(sections+1-i)
  # Create the atom to mark
  obj1 = BuildAtom Carbon
  ColorObj (obj1),(color)
  BallObj (obj1)
  # Create the donut around it
  obj2 = ShowTorus LargeRadius=2,LargeEdges=32,SmallRadius=0.5,SmallEdges=10,Color=(color)
  NameObj (obj2),'Option(i)'
  PosObj (join obj),-28,(y),(z)
  # Create the text
  Font Arial,Height=2,Color=(color),Depth=1.0,DepthCol=800000
  PosText 0%,(100.*(sections-i)/(sections-1))%,Left
  Print (sectionlist(i))
PrintCon
PosObj (textobj),8,0,(z)
JoinObj Methane,4
NumberObj all,1
# Load the menu
MoveObj not HMD Yami,X=-100,0,15
AutoMoveObj not HMD Yami,X=2
Wait 50
AutoMoveObj not HMD Yami,X=0
# Wait until user selects an option
ShowMessage "Please click one circle to get specific usage instructions..."
# Take snapshot here, Eliza!
waits=0
do
  Wait 2
  opt = MarkAtom
  waits=waits+1
  if (waits>100)
    HideMessage
while opt<Atoms-sections+1
HideMessage
# The last atoms in the soup are our options, so opt is now 1..9
opt=opt-Atoms+sections
# Let options fly away
AutoRotateObj Option(opt),X=1,Y=3,Z=2
Wait 60
AutoRotateObj Sections,Y=1
AutoMoveObj Sections,Z=-2
AutoMoveObj Option? Methane,Y=-1
Wait 100
# Go to selected section
Go Section(opt)

#
# Section 1 - VR setup
#
Section1:
StartSection
textlist='(sectionlist1)',
         "Welcome to Yet Another Scientific Virtual Reality Application!",
         "This help movie will guide you through all necessary steps to dive right into the nano-scale world!",
         "Let's first have a look at this flowchart showing how to set up your VR system."
ShowTitleAndMessages textlist
HideMessage
# Show VR system setup flowchart
flw=LoadPNG flowchart
flwwidth=2048
flwheight=1152
for i=1 to 100
  ShowImage (flw),0,0,(flwwidth),(flwheight),(i)
  MoveObj HMD Yami,0,-1
  Wait 1
PrintImage (flw)
# Open manual for detailed setup instructions
Font Baikal,Height=(flwheight*.02)
ShowButton 'Click here for detailed instructions!',(flwwidth*.8),(flwheight*.15),Color=ff8080,Action=ShowURL ("file://"+YASARADir+"/doc/RYivrmoP.html")
PrintCon
Wait ContinueButton
for i=1 to 100
  ShowImage (flw),0,0,(flwwidth),(flwheight),(100-i)
  MoveObj HMD Yami,0,1
  Wait 1
DelImage (flw)
# Switch on controllers
textlist="Now that your VR system is installed, let's see if we can activate the controllers:",
         "Click Window > Stereo > Virtual reality parameters > Enable controllers without VR headset > OK. ",
         "Make sure that your controllers are switched on, but don't be disappointed if you cannot see them yet!"
ShowMessages textlist
VrPar On
if Headset=='Vive' or Headset=='Vive Pro' or Headset=='Vive Pro 2' or Headset=='Vive Cosmos Elite' or Headset=='Valve Index'
  ShowMessage "Make sure that the controllers are unobstructedly seen by the base stations."
else
  if Headset=='Meta Quest 2'
    textlist="The Quest will per default go to sleep mode after 15 seconds when not worn on your head.",
             "Inside your Quest go to Settings > System > Power and set 'Auto Sleep Headset' to 4 hours."
    ShowMessages textlist
  ShowMessage "Make sure to turn the front of the headset with its tracking cameras towards the controllers."
Wait ContinueButton
ShowMessage "Next we will calibrate your monitor to determine its position in VR space."
Wait ContinueButton
# Calibrate screen corners
cornerlist()='TopLeft','BottomLeft','TopRight','BottomRight'
orilist()='30,60','-30,60','30,-60','-30,-60'
for i=1 to count cornerlist
  corner=cornerlist(i)
  ShowMessage 'Please put the controller on the (corner) screen corner, then pull the trigger shortly.'
  AutoOri (orilist(i)),0,20,0
  # Copy last click position to screen corner
  VRPar ScrCorner=(corner)
  # Remember the start parameters
  startparlist() = VRPar
  while 1
    # Copy last click position to screen corner
    VRPar ScrCorner=(corner)
    # Did it change?
    parlist() = VRPar
    if parlist(count parlist-14+i*3)!=startparlist(count parlist-14+i*3)
      break
    Wait 1
AutoOri 0,0,0,20,0
textlist="Done! The controllers should be now visible on screen.",
         "In the next section we will see how to use VR controllers as a 3D mouse.",
         "Press 'Continue' to proceed to the next section..."
ShowMessages textlist
AutoPosOri 25,0,70,0,30,0,15,1

#
# Section 2 - VR controllers
#
Section2:
VRPar On
StartSection
textlist='(sectionlist2)',
         "After calibrating the screen in Section 1, let's see how to use a controller as 3D mouse.",
         "But first of all, let's have a look at two important parameters: Controller movement Z-scale and Z-shift.",
         "These two parameters can be set in 'Window > Stereo > Virtual reality parameters'.",
         "'Controller shift along Z' (in centimeters) moves the controller from its real space position in front of the monitor,",
         "along YASARA's global Z-axis behind the monitor, so that we can actually see the controller.",
         "'Controller move scale Z' scales up the movement along YASARA's global Z-axis, so that objects far away can be easily moved.",
         "E.g. a factor of 10 turns a comfortable arm's reach of 30 cm into a three meter working space.",
         "Experiment now with these parameters in 'Window > Stereo > Virtual reality parameters' and press 'Continue' when you are done.",
         "For more information about the virtual reality parameters, click 'Help > Show command docs' and type 'VRPar'."
ShowTitleAndMessages textlist
ShowMessage "Let's start using the controllers! You can use one controller in your left hand and still use the mouse with your right one."
AutoPos 0,0,200,50,1
Wait ContinueButton
UserInput On
textlist="Hold the grip button on the controller side to move the scene. Yami will now follow the controller and imitate its rotation.",
         "Try to turn Yami around and on his head!",
         "Bring the controller in touch with Yami and push the grip button to move him as if you held him in your hand.",
         "Try now to grab Yami's foot! You should feel a light rumble when you grab a scene object tightly.",
         "Press 'Continue' to proceed to the next section..."
ShowMessages textlist
UserInput Off
AutoPosOri 25,0,70,0,30,0,15,1

#
# Section 3 - VR screen
#
Section3:
StartSection
ShowMessage "Time to dive into VR! Click 'Window > Stereo > Virtual reality', then put on your headset."
do
  Wait 2
  vron=Stereo
while !vron
HideImage all
textlist='(sectionlist3)',
         "Welcome in VR! Take a moment to look around. You should see your controllers if they are switched on.",
         "In front of you is a virtual representation of your monitor. Try to touch its edges with your controllers.",
         "Now adapt the height. While we normally look down on our screen, it needs to be higher in VR, otherwise it will obstruct the view.",
         "Click 'Window > Stereo > Virtual reality parameters > Screen shift along Y' and place the screen center in front of your eyes.",
         "And now it's time for a free screen upgrade: Click 'Window > Stereo > Virtual reality parameters > Screen scale'.",
         "The monitor also defines YASARA's global coordinate system:",
         "The X-axis is parallel to the horizontal, and the Y-axis is parallel to the vertical monitor edge.",
         "The Z-axis goes into the screen and the origin with the coordinates 0/0/0 is in the screen center.",
         "To see this click 'Effects > Position > Set for scene > OK' to place the scene in the screen center."
ShowTitleAndMessages textlist
HideMessage
# Position everything in the center to demonstrate the origin
Pos 0,0,0
# Move a bit away and and show Yami's eyes
AutoPos 0,0,20,Steps=20,Wait=Yes
hmdposori()=PosOriObj HMD
hmdposori2=hmdposori2+7
hmdposori4=hmdposori4-30
AutoPosOriObj HMD,(hmdposori),Steps=10,Wait=Yes
textlist="Atoms can be marked as usally by left-clicking on them with the mouse, but it gets a bit tricky in 3D:",
         "70% of all humans have a dominant right eye and can click on atoms intuitively,",
         "the other 30% have to mentally focus on their right eye or simply close the left one."
ShowMessages textlist
ShowMessage "Now try to mark Yami's left or right eye"
do
  markedatom=MarkAtom
  Wait 2
while markedatom!=1009 and markedatom!=1010
textlist="Well done! By the way, the graphical user interface (GUI) is not fixed to the screen...",
         'Press the (buttonmenu) on the VR controller to move it between marked atom, controllers, free space and VR screen.',
         "The size of the portable GUI can be set at 'Window > Stereo > Virtual reality parameters > Ctrl./atom GUI scale'.",
         'Return the GUI back to the monitor by pressing the (buttonmenu) multiple times.'
ShowMessages textlist
UserInput On
textlist="The scene and objects in the scene can be moved and rotated with the mouse just as you always do in YASARA:",
         "Hold the left button to rotate, right button to move along Z and both (or middle) button(s) to move along XY...",
         "Click the lasso selection icon at the top, draw a selection and note how moving your head affects the selection."
ShowMessages textlist
UserInput Off
UnselectAll
AutoOri 0,0,0,10,1
hmdposori()=PosOriObj HMD
hmdposori2=hmdposori2-7
hmdposori4=hmdposori4+30
AutoPosOriObj HMD,(hmdposori),Steps=10,Wait=Yes
AutoPosOri 25,0,70,0,30,0,15,1

#
# Section 4 - VR playground
#
Section4:
Stereo VR
StartSection
# Load pillar
tex=LoadJPG pillar.xei
pil=LoadWOb pillar,Open=No,(tex)
ScaleObj (pil),1,.5,1
# Load and style E coli AlkB bound to dsDNA
pdb=LoadPDB 3bie,Download=Yes
DelRes Water
Clean
Style Ribbon,Stick
HideTrace all
ShowAtom NucAcid
BallAtom NucAcid Backbone
BallStickAtom NucAcid Sidechain
ShowSurfObj (pdb),Molecular
cut=CutObj (pdb)
SwitchObj (pil) (pdb) (cut),off
# Load Yami again to undo style changes
yamipos()=PosObj Yami
DelObj Yami
LoadYOb yami
NameObj yami,Yami
PosObj Yami,(yamipos)
TransferObj HMD,Yami,Fix
UserInput On
textlist='(sectionlist4)',
         "It's time to explore the virtual world around you!",
         "Since we will leave your mouse behind, we need to navigate the GUI with the VR controllers.",
         '(buttoncenterclick) to press the "Continue" button without using your mouse.',
         "Take a controller and pull the trigger lightly. This will activate a laser beam.",
         "The beam angle can be set at 'Window > Stereo > Virtual reality parameters > Beam angle'.",
         "Pulling the trigger fully and shortly has the same effect as a left mouse click.",
         "Point the beam on the 'Continue' button and click it by pulling the trigger shortly!",
         "Well done! Before we start exploring, remember to take the portable GUI with you...",
         '...by pressing the (buttonmenu). Now click "Continue" with your second controller.'
ShowTitleAndMessages textlist
HideMessage
HUD off
# Show protein on pillar, assume user moved HMD which initalizes the chaperone so VRArea returns sensible values
width,height,beta,pilpos()=VRArea
PosOriObj (pil),(pilpos),0,(beta),0
PosOriObj (pdb),(pilpos),45,(beta-90),55
MoveObj (pdb),0,72
OriObj (cut),Beta=(beta)
SwitchObj (pil) (pdb),On
LabelRes CA Res LYS 166,"E. coli AlkB bound to dsDNA",Height=2,Yellow,Y=-10
# Let Yami jump to the pillar
yamiposori()=pilpos1+width/2*.8*sign pilpos1,pilpos2+22,pilpos3-height*.8/2*sign pilpos3,0,45*sign pilpos1,0
jumpposori()=yamiposori/2
jumpposori2=100
AutoPosOriObj Yami HMD,(jumpposori),Steps=100,Wait=Yes
AutoPosOriObj Yami HMD,(yamiposori),Steps=100,Wait=Yes
textlist="Stand up, push back your chair and take a look around you!",
         "On the floor you can see a grid - the VR area from the SteamVR room setup.",
         "To avoid stumbling over the chair, attach a Vive Tracker to the backrest to see it in VR.",
         "On top of the pillar you can see the DNA alkylation damage repair protein AlkB.",
         "Remember to stay inside the grid to be safe and not fall into the void of empty space!",
         "The light source is attached to your HMD, hence there are no shadows.",
         "You can actually see the light reflection when looking at your controller.",
         "Labels always face you, so that they are readable from all directions.",
         "Try to mark an atom by pointing the beam at it and clicking the trigger.",
         "You can also navigate the GUI by pointing the beam and pulling the trigger:",
         "Activate the HUD by clicking 'Window > Head-up display > Scene content'."
ShowMessages textlist
HUD Obj
textlist="Objects can be switched on/off by trigger-clicking them in the VIS column.",
         '(buttonlefthold) and trigger-click to mark up to four atoms...',
         "...and show the distance, angle and dihedral between them in the HUD.",
         '(buttonrightclick) to open the context menu of marked&selected atoms.',
         '(buttonrighthold) and trigger-click to do this in one step.',
         "In the same way you can open the context menu in the sequence selector...",
         "...and switch one object on and all others off in the VIS column.",
         "Click the box or lasso selection icons and pull the trigger to create a selection.",
         "Try to focus on your right eye or close the left one to make a precise selection.",
         "To grab a single object for movement, point the beam at the protein...",
         "...and pull the trigger fully until the rotating sphere appears.",
         "Hold the grip button to lift the protein, which follows the controller loosely...",
         "...and imitates its rotation. The farther away the object, the faster it moves.",
         "Hold the grip button and pull the trigger lightly to move without rotation.",
         "Hold the grip button and pull the trigger fully to swing the object around.",
         "Touch an object with your controller and hold grip to take it in your hand.",
         '(buttonlefthold) and grip to rotate an object without moving it.',
         '(buttondownclick) to put a grabbed object on the controller.',
         '(buttonupclick) to send it back to its original position.',
         '(buttonuphold) to move the grabbed object away along the beam.',
         '(buttondownhold) to move the object towards the controller.',
         "Press Grip shortly to release an object from the top of the controller.",
         "Trigger-click in the VIS column to switch the 'CutPlane' object on."
ShowMessages textlist
SwitchObj CutPlane,on
textlist='(buttonlefthold) and trigger-click the CutPlane object in the HUD to grab it.',
         "Move and rotate the cut plane with the controller to adapt the surface.",
         "Point the beam into nowhere and pull the trigger fully to grab the whole scene.",
         "Finally, try to sit down on your chair again. With the Vive Tracker, it's easy.."
ShowMessages textlist

#
# Section 5 - VR keyboard
#
Section5:
Stereo VR
StartSection
UserInput off
# Load 1AON
obj=LoadPDB 1aon
BallObj (obj)
for mol='A' to 'U'
  ColorMol (mol) Obj (obj),((ord mol-65)*17)
PosObj (obj),0,0,0
RotateAtom Obj (obj),X=180
# Split into molecule objects
objlist() = SplitObj (obj)
# Join the Mg atoms and ATP molecules to their objects
for i=1 to 7
  JoinObj (objlist(i+21)),(objlist(i))
  JoinObj (objlist(i+28)),(objlist(i))
# Get the positions of the objects
objs = CountObj 1aon?
poslist() = PosObj 1aon?
# Move them outwards and give them a random rotation
scale=5
for i=1 to objs
  obj=objlist(i)
  PosObj (obj),(poslist(i*3-2)*scale),(poslist(i*3-1)*scale),(poslist(i*3)*scale)
  AutoRotateObj (obj),(rnd 1-0.5),(rnd 1-0.5),(rnd 1-0.5)
textlist='(sectionlist5)',
         "The proteins rotating around you are the 21 building blocks of GroEL/ES.",
         "GroEL/ES is a chaperonin that helps folding proteins by encapsulating them.",
         "Look around and try to find them all. They are arranged in three rings.",
         "If you press 'Continue', you will be encapsulated yourself. Get ready to fold..."
ShowTitleAndMessages textlist
# Encapsulate the viewer
HideMessage
for i=1 to objs
  obj=objlist(i)
  steps=1000+rnd 1000
  AutoPosOriObj (obj),(poslist(i*3-2)),(poslist(i*3-1)),(poslist(i*3)+8),0,0,0,Steps=(steps),Wait=No
Wait 2000
UserInput on
ShowMessage "Can you blindly press keys <F1> to <F8> to change the atom display style?"
Wait ContinueButton
if Headset=='Vive' or Headset=='Vive Pro' or Headset=='Vive Pro 2' or Headset=='Valve Index'
  textlist="If not, look at your keyboard in front of your monitor!",
           "You should see a video stream from your front camera blended in, showing your real keyboard.",
           "If this is not the case, exit YASARA, start SteamVR and go to 'Settings > Camera > Enable camera'.",
           "To read the letters on the keys, buy a backlit keyboard with large letters.",
           "Check that the camera image is aligned correctly with the VR world. Put a controller on the keyboard...",
           "...then click 'Window > Stereo > Virtual reality parameters > Calibrate cameras' and shift the sliders.",
           "For stereo cameras, align the real and the VR controller once for each eye, keeping the other eye closed.",
           'For a mixed reality experience, hold the (buttonmenu) on your controller, but beware of motion sickness!'
else
  if Headset=='Meta Quest 2'
    textlist()="If not, double tap the side of your headset to active and deactivate pass-through."
  elif Headset=='Vive Cosmos Elite'
    textlist="If not, you first need the VIVE Console App to be installed.",
             "Then go in your VIVE Console App to Settings > Advanced > Headset button and select Pass-through.",
             "Now push the button on the side of your headset to active and deactivate pass-through."
  else
    textlist()="If not, activate pass-through on your headset."
  ShowMessages textlist
  textlist="In your SteamVR window activate 'Display VR View' and maximize the window.",
           "Now you should be able to see your keyboard as well as what you type.",
           "To read the letters on the keys, buy a backlit keyboard with large letters."
ShowMessages textlist
textlist="In case you need to type text when standing, click the gray boxes on the left or right side of the GUI.",
         "Click the left box to press multiple keys, and the right box for a single key. The VR keyboard shows up.",
         "Press the top right 'Hide' key to hide the virtual keyboard again.",
         "To facilitate typing in the console, the input is also shown enlarged at the top of your screen.",
         "The input scaling and other parameters can be set in 'Options > Input devices > Virtual keyboard > Parameters'.",
         "Sit down again and press 'Continue' to proceed to the next section..."
ShowMessages textlist
HideMessage
# Let Yami fly away in its GroEL rocket
for i=1 to 100
  AutoMove Z=(i)
  Wait 5

#
# Section 6 - HIV protease
#
Section6:
Stereo VR
StartSection
textlist='(sectionlist6)',
         "After all the VR fun, let's proceed with a more scientific example: drug design",
         "What follows is a shortened VR version of the help movie '4.5. Drug design'."
ShowTitleAndMessages textlist
HideMessage
# Let Yami vanish
cut=Cut
RotateObj (cut),90
MoveObj (cut),Z=-100
AutoRotateObj (cut),0,1
AutoMoveObj (cut),Z=.2
Wait 360
AutoRotateObj (cut),0,0
AutoMoveObj (cut),Z=0
Wait 10
DelObj Yami (cut)
_,_,_,_,floory=VRArea
AutoRotate 0,0,10
Wait 10
AutoRotate 0,0,-10
Wait 20
AutoRotate 0,0,10
Wait 20
AutoRotate 0,0,-10
Wait 20
AutoRotate 0,0,10
Wait 10
AutoRotate 0,0,0
Wait 20
AutoPosObj HMD,Y=(floory),Steps=20
DelAll
# Start drug design
Style Ribbon,Off,BallStick
StickRadius 60%
Boundary wall
SurfPar Probe=1.4,Resolution=3,Molecular=Numeric,ESPMax=200.0,SmoothCut=Yes,Radii=Solvation
ColorSurfAll OutCol=AtomCol,OutAlpha=85,InCol=Green,InAlpha=60,Specular=Yes
textcom="Molecular modeling in VR works best with the mouse in your main hand and the controller in the other.",-1,'pass',
        "Click 'File > Load > PDB file' and double-click '3i7e' to load HIV protease with TMC-126.",0,'LoadPDB 3i7e',
        "Since X-ray structures usually don't resolve hydrogens, these need to be added.",-1,'pass',
        "Click 'Edit > Add > Hydrogens to Object', double-click '3i7e'.",1655,'AddHyd',
        "To visualize interactions, you would normally click 'View > Style scene > Ribbon&ligand'.",-1,'pass',
        "This option does not work in movies, so it will be done automatically now...",-1,'ShowRibbonAndLigand',
        "Green lines are hydrophobic interactions, style them at 'Effects > Point&line appearance'.",-1,'pass',
        "Let's create a view that shows only the active site.",-1,'pass',
        "Click 'View > Duplicate', type 'Active site', on the real or virtual keyboard.",-4,'DuplicateView Main,"Active Site"',
        "Let's have a closer look: hold the grip button and pull the complex towards you.",-1,'pass',
        "There are two different ways to select surrounding residues.",-1,'pass',
        "First those within a distance from the ligand center: mark any atom in the ligand.",-1,'pass',
        '(buttonrightclick) or right-click with the mouse to open the context menu.',-1,'pass',
        "Pick 'Select > in sphere around > Residue', pull the trigger and move the controller to adapt the selection sphere radius.",-1,'pass',
        "Alternatively, select those residues within a distance from any ligand atom:",-1,'pass',
        "Click 'Edit > Select > New > Residue', pick 'All' in the right list and as manual selection type: with distance<5 from djr",-1,'SelectRes all with distance<5 from DJR',
        "By default, the Ribbon & Ligand style shows only atoms involved in interactions.",-1,'MarkAtom None',
        "Make sure that no atom is marked, right-click to open the selection context menu and choose 'Show > Atoms'.",-1,'ShowAtom selected',
        "Click 'View > Hide atoms > Aliphatic hydrogens' to make the view less crowded.",-1,'HideAtom Element H with bond to Element C',
        "The water molecules are not really needed, so click 'Edit > Delete > Waters'.",-1,'DelWater',
        "In the next step, let's create a partial surface...",-1,'pass',
        "Click 'Edit > Add > To surface environment: Residue'. Delete the 'with distance<5..' you typed before and double-click 'Protein'.",-1,'AddEnvRes Protein',
        "Click 'View > Show surface > Residue', choose 'Selected' and (with Ctrl) 'Protein', set colors 'AtomCol' and inside 'Green'.",-1,'ShowSurfRes Protein Selected,Type=Molecular,Update=Dynamic,OutCol=AtomCol,OutAlpha=85,InCol=Green,InAlpha=60,Specular=Yes',
        "Now the surface has opacity 85 when seen from outside, and 60 from inside.",-1,'pass',
        "To get rid of the rest, click 'Edit > Select > Invert', now the rest is selected.",-1,'MarkAtom None',
        "Right-click to open the selection context menu and choose 'Hide > Secondary structure'.",-1,'HideSecStrRes all with distance>5 from DJR',
        "To predict the affinity of this ligand, check help movie 4.5, 'Drug design'",-1,'pass',
        "To improve the ligand, continue with section 9, the 'Virtual reality simulations' help movie.",-1,'pass',
        "Press 'Continue' to proceed to the next section...",-1,'pass'
ShowTutorial textcom
UnselectAll
DelView 2

#
# Section 7 - The screening dome
#
Section7:
Stereo VR
StartSection
UserInput On
textlist='(sectionlist7)',
         "Let's load a virtual screening result from YASARA's dock_runscreening macro:",
         "Click 'File > Load > YASARA Scene' and double-click '4emn_screening.sce'."
ShowTitleAndMessages textlist
LoadSce '4emn_screening'
objs=CountObj All
_,_,_,_,_,zpos=VRArea
textlist='This is the catalytic domain of the resuscitation promoting factor B docked with (objs) FDA approved drugs.',
         "RpfB plays a crucial role in the revival of Mycobacterium tuberculosis from dormancy (Ruggiero et al., 2013).",
         "It's hard to see anything as they all sit on top of each other, so let's disperse them:",
         'Click "Effects > Position > Set for object automatically", double-click "All" in the right list, choose position 0/0/(0+zpos), TopHemisphere and radius 150 A.'
ShowMessages textlist
AutoPosObj All,0,0,(zpos),100,Yes,TopHemisphere,150
LabelRes Mol B,COMPOUND,3,Yellow,Y=-10
textlist='Now stand up and press the (buttonmenu) to take the GUI with you.',
         "Point at an object with the laser, grab it by pulling the trigger until a sphere appears.",
         '(buttondownclick) to put the grabbed object on the controller.',
         '(buttonupclick) to send that object back. Look at a few complexes with both hands.',
         "Choose one complex for examination, grab it and pull it closer with the grip button.",
         "Mark any atom in the object and split it in the atom context menu:",
         '(buttonrightclick) and click "Split > Object > Split at all molecules".',
         "Now the protein and the ligand are separate objects.",
         "Put the receptor on the first controller, the ligand on the other, and detach the GUI.",
         "Dock the ligand manually, press the grip buttons to release each object.",
         "Touch each object with a controller and hold grip buttons to take them in your hands.",
         "If you want to see interactions while docking, do it during an MD, see last section.",
         "Sit down again and press 'Continue' to proceed to the next section..."
ShowMessages textlist

#
# Section 8 - Teleportation
#
Section8:
Stereo VR
StartSection
UserInput On
textlist='(sectionlist8)',
         "Lets's explore a giant model from the YASARA www.PetWorld.vision database in VR!",
         "Click 'File > Load > YASARA Scene', double-click 'sarscov2_2.sce'."
ShowTitleAndMessages textlist
if Objects<8
  LoadSce sarscov2_2.sce
textlist="Such large structures take the GPU to the limits, but there is an easy trick:",
         "Click 'Edit > Delete > Hydrogens' to reduce the atom number, then stand up with the GUI.",
         "If you use the VR controllers to pull large objects close, like this SARS-CoV-2 model...",
         "...this can become shaky and induce motion sickness. Most VR apps use teleportation instead.",
         "Point the beam onto the floor grid and pull the trigger fully to activate the teleporter.",
         "The teleporter appears as a rectangle around the floor grid. Can you see it?",
         "It can be grabbed and moved like any other object. Move the teleporter to the virus surface.",
         '(buttonupclick) to teleport your workplace to the new location.',
         "Try to move the teleport again somewhere else. Note that you can turn it around.",
         '(buttondownclick) to bring the teleporter back instead of using it.',
         '(buttondownclick) a second time to deactivate the teleporter.',
         "Now take the nasty virus apart: grab various objects and pull them out.",
         "Sit down again and press 'Continue' to proceed to the next section..."
ShowMessages textlist

#
# Continue to next help movie
#
Section9:
StartSection
textlist='(sectionlist9)',
         "Finally, we will explore MD simulations in VR!",
         "Press 'Continue' to start the YASARA help movie: Virtual reality simulations (PC)"
ShowTitleAndMessages textlist
PlayMovie '../vrintro3/vrintro3.mcr'

# MOVE YAMI ALONG
# ===============
# x IS THE X-POSITION, beta THE BETA ORIENTATION ANGLE, tilt THE AMOUNT OF TILTING (~SPEED),
# step IS THE ANIMATION STEP
def AnimateYami x,beta,tilt,step
  step=step%60
  # Rotate Yami's antennas
  Dihedral N Res Lys,CA Res Lys,CB Res Lys,CG Res Lys Obj Yami,bound=Yes,set=(6*step-180)
  # Place Yami
  height=sin (180.*step/60)*18
  PosObj Yami,(x),(height-20),150
  OriObj Yami,0,0,0
  RotateObj Yami,X=(-sin (360.*step/60)*30*tilt),Y=(beta)

# LOAD YAMI WITH HMD
# ==================
def LoadYami
  yami=LoadYOb yami
  NameObj (yami),Yami
  tex=LoadJPG generic_hmd
  hmd=LoadWOb generic_hmd,Open=No,(tex)
  NameObj (hmd),HMD
  scale=75
  ScaleObj HMD,(scale),(scale),(scale)
  PosOriObj HMD, X=-1.601, Y=4.441, Z=41.296, Alpha=8.418, Beta=-168.021, Gamma=-3.742

# PREPARE SCENE FOR NEW SECTION
# =============================
def StartSection
  global bgr,buttonmenu,buttonleftclick,buttonlefthold,buttonrightclick,buttonrighthold,buttonupclick,buttonuphold,buttondownclick,buttondownhold,buttoncenterclick
  # Reset scene
  DelAll
  DelImage not (bgr)
  HideImage (bgr)
  vron=Stereo
  if !vron
    ShowImage (bgr)
  LoadYami
  PosOri 25,0,70,0,30,0
  AutoPosOri 0,0,50,0,0,0,50,1
  # Assign button descriptions depending on headset
  buttonupclick='Tap joystick up'
  buttonuphold='Hold joystick up'
  buttondownclick='Tap joystick down'
  buttondownhold='Hold joystick down'
  buttoncenterclick='Click the joystick'
  if Headset=='Vive' or Headset=='Vive Pro' or Headset=='Vive Pro 2' or Headset=='Vive Cosmos Elite'
    # Vive
    buttonmenu='Menu button'
    buttonleftclick='Click trackpad left'
    buttonlefthold='Hold trackpad left'
    buttonrightclick='Click trackpad right'
    buttonrighthold='Hold trackpad right'
    buttonupclick='Click trackpad up'
    buttonuphold='Hold trackpad up'
    buttondownclick='Click trackpad down'
    buttondownhold='Hold trackpad down'
    buttoncenterclick='Click trackpad in the center'
  elif Headset=='Valve Index'
    # Valve Index
    buttonmenu='B button'
    buttonleftclick='Click left A or tap joystick left'
    buttonlefthold='Hold left A or hold joystick left'
    buttonrightclick='Click right A or tap joystick right'
    buttonrighthold='Hold right A or hold joystick right'
  else
    # Meta Quest 2 and any other headset
    buttonmenu='Y or B button'
    buttonleftclick='Click X or tap joystick left'
    buttonlefthold='Hold X or hold joystick left'
    buttonrightclick='Click A or tap joystick right'
    buttonrighthold='Hold A or hold joystick right'

# SHOW A TUTORIAL
# ===============
def ShowTutorial textcom()
  # Delete leftovers
  UserInput On
  # Hidden command to keep undo/redo active during movies, helps in case of building accidents
  KeepTab
  # Show the messages
  for i=1 to count textcom step 3
    ShowMessage (textcom(i))
    # textcom(i+1) IS -1 TO EXECUTE ALWAYS, -2 TO EXECUTE ONLY IF NO CELL IS PRESENT, -3 NOT TO WAIT,
    if textcom(i+1)!=-3
      Wait ContinueButton
    cellobj = ListObj SimCell
    if textcom(i+1)==Atoms or textcom(i+1)==-1 or (textcom(i+1)==-2 and !cellobj) or textcom(i+1)<-2 and textcom(i+1)!=-4
      # Number of atoms unchanged, assume that user just clicked 'Continue', or always execute
      (textcom(i+2))
    if textcom(i+1)==-4
      # Create second view if missing
      views=DuplicateView Main,tmp
      DelView tmp
      if views==2
        (textcom(i+2))
      ShowView 2
    if textcom(i+1)==-5
      # Switch to faster molecular surface for interactive dihedral changes+minimization
      SurfPar Molecular=Gaussian
    if textcom(i+1)==-6
      # Prepare minimization
      TempCtrl SteepDes
      Sim On
      Wait SpeedMax<3000
      Sim Off  

# SHOW A LIST OF TEXT MESSAGES
# ============================
def ShowMessages textlist()
  for i=1 to count textlist
    ShowMessage (textlist(i))
    Wait ContinueButton

# SHOW TITLE AND A LIST OF TEXT MESSAGES
# ======================================
def ShowTitleAndMessages textlist()
  vron=Stereo
  if vron
    scale=20
    startpos=0,300,100
    titlepos=0,50,100
    endpos=0,-500,100
  else
    scale=1
    startpos=0,54,10
    titlepos=0,4,10
    endpos=0,-46,10
  # Use 'Title' instead of the object number, the user may load a scene in the mean time
  DelObj 'Title'
  MakeTextObj 'Title',Width=(30*scale),Height=(3*scale)
  Font Arial,Height=(1*scale),Color=Yellow,Depth=(0.5*scale),DepthCol=800000
  Print (textlist1)
  PrintCon
  PosObj Title,(startpos)
  AutoPosObj Title,(titlepos),50,0
  ShowMessage (textlist2)
  Wait ContinueButton
  AutoRotateObj Title,X=-1
  AutoPosObj Title,(endpos),100,0
  ShowMessage (textlist3)
  Wait ContinueButton
  DelObj Title
  for i=1 to 3
    DelVar textlist1
  ShowMessages textlist

# Show and style the ligands
# ==========================
def ShowRibbonAndLigand
  Style Ribbon
  StickAll
  # List of interaction types to visualize, and if occluded interactions should be considered
  intlist='Hydrophobic','PiPi','CationPi','Ionic'
  occlist='No',         'Yes', 'Yes',     'Yes' 
  # Delete contact objects from previous runs
  DelObj (join intlist)
  # Get a list of objects with atoms
  objlist() = ListObj Atom all
  # Loop over the objects
  for obj in objlist
    # Does the object contain hydrogens?
    hydrogens = CountAtom Element H Obj (obj)
    if !hydrogens
      # No, but we need them to display hydrogen bonds and find interactions, add them
      AddHydObj (obj)
    # Find the largest hetgroup with >6 atoms, skip water and ions right away 
    # Changes here must be duplicated in md_analyze.mcr
    reslist() = ListRes Hetgroup !Water Obj (obj) with >0 bonds to all
    reslenmax=6
    ligandname=''
    carbohydlist()=''
    for res in reslist
      reslen = CountAtom (res)
      resname = NameRes (res)
      carbons = CountAtom Element C (res) with bond to Element O
      if carbons>4 and not (resname in carbohydlist)
        carbohydlist(1+count carbohydlist)=resname
      if reslen>reslenmax
        reslenmax=reslen
        ligandname=resname
    if ligandname!=''
      if ligandname in carbohydlist
        # Ligand is a carbohydrate, treat all carbohydrate residues as ligands
        ligandname=join carbohydlist
      # Now 'ligand' selects e.g. all ATP residues in object 5: 'Res ATP Obj 5'
      ligand='Res (ligandname) Obj (obj)'
      # This object contains a ligand
      # Show the ligand
      BallStickRes (ligand)
      ShowRes (ligand)
      if Model
        # In YASARA Model+, we can use interaction analysis to determine which protein residues to show
        # Get a list of protein atoms that are H-bonded to the ligand
        atomlist() = ListHBoAtom Obj (obj),(ligand)
        # Show protein backbone NH/C=O groups involved in H-bonds
        ShowAtom H C with bond to (join atomlist) or (join atomlist)
        for i=1 to count intlist
          # Get a list of protein atoms involved in other interactions, and join to atomlist
          intatomlist() = ListIntAtom Obj (obj),(ligand),Type=(intlist(i)),Cutoff=5,Occluded=(occlist(i))
          atomlist()=atomlist,intatomlist
        # Show all Sidechain & CA atoms of residues with atoms involved in interactions
        if count atomlist
          ShowAtom Sidechain CA Res Atom (join atomlist)
      else
        # In YASARA View, we need to simply show all residues around, i.e. 5 A from ligand 
        ShowRes Obj (obj) with distance<5 from (ligand)
      # Show all hydrogen bonds in the ligand, and between ligand and protein
      ShowHBoRes (ligand),Extend=Yes
      # Show hydrophobic, pi-pi and cation-pi interaction
      for i=1 to count intlist  
        ShowIntAtom (ligand),not (ligand) and Obj (obj),Type=(intlist(i)),Cutoff=5,Occluded=(occlist(i))
  # Hide non-polar hydrogens again
  HideAtom Element H with bond to Element C
